# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/musy',
                'doc_host': 'https://CarloLepelaars.github.io',
                'git_url': 'https://github.com/CarloLepelaars/musy',
                'lib_path': 'musy'},
  'syms': { 'musy.chord': { 'musy.chord.Chord': ('chord.html#chord', 'musy/chord.py'),
                            'musy.chord.Chord.__add__': ('chord.html#chord.__add__', 'musy/chord.py'),
                            'musy.chord.Chord.__eq__': ('chord.html#chord.__eq__', 'musy/chord.py'),
                            'musy.chord.Chord.__floordiv__': ('chord.html#chord.__floordiv__', 'musy/chord.py'),
                            'musy.chord.Chord.__ge__': ('chord.html#chord.__ge__', 'musy/chord.py'),
                            'musy.chord.Chord.__gt__': ('chord.html#chord.__gt__', 'musy/chord.py'),
                            'musy.chord.Chord.__index__': ('chord.html#chord.__index__', 'musy/chord.py'),
                            'musy.chord.Chord.__init__': ('chord.html#chord.__init__', 'musy/chord.py'),
                            'musy.chord.Chord.__int__': ('chord.html#chord.__int__', 'musy/chord.py'),
                            'musy.chord.Chord.__iter__': ('chord.html#chord.__iter__', 'musy/chord.py'),
                            'musy.chord.Chord.__le__': ('chord.html#chord.__le__', 'musy/chord.py'),
                            'musy.chord.Chord.__len__': ('chord.html#chord.__len__', 'musy/chord.py'),
                            'musy.chord.Chord.__lt__': ('chord.html#chord.__lt__', 'musy/chord.py'),
                            'musy.chord.Chord.__mod__': ('chord.html#chord.__mod__', 'musy/chord.py'),
                            'musy.chord.Chord.__mul__': ('chord.html#chord.__mul__', 'musy/chord.py'),
                            'musy.chord.Chord.__ne__': ('chord.html#chord.__ne__', 'musy/chord.py'),
                            'musy.chord.Chord.__repr__': ('chord.html#chord.__repr__', 'musy/chord.py'),
                            'musy.chord.Chord.__sub__': ('chord.html#chord.__sub__', 'musy/chord.py'),
                            'musy.chord.Chord._compare_notes': ('chord.html#chord._compare_notes', 'musy/chord.py'),
                            'musy.chord.Chord.abs_intervals': ('chord.html#chord.abs_intervals', 'musy/chord.py'),
                            'musy.chord.Chord.from_short': ('chord.html#chord.from_short', 'musy/chord.py'),
                            'musy.chord.Chord.get_audio_array': ('chord.html#chord.get_audio_array', 'musy/chord.py'),
                            'musy.chord.Chord.invert': ('chord.html#chord.invert', 'musy/chord.py'),
                            'musy.chord.Chord.midi': ('chord.html#chord.midi', 'musy/chord.py'),
                            'musy.chord.Chord.name': ('chord.html#chord.name', 'musy/chord.py'),
                            'musy.chord.Chord.names': ('chord.html#chord.names', 'musy/chord.py'),
                            'musy.chord.Chord.play': ('chord.html#chord.play', 'musy/chord.py'),
                            'musy.chord.Chord.rel_intervals': ('chord.html#chord.rel_intervals', 'musy/chord.py'),
                            'musy.chord.Chord.to_frame': ('chord.html#chord.to_frame', 'musy/chord.py'),
                            'musy.chord.Note.__mul__': ('chord.html#note.__mul__', 'musy/chord.py'),
                            'musy.chord.PolyChord': ('chord.html#polychord', 'musy/chord.py'),
                            'musy.chord.PolyChord.__init__': ('chord.html#polychord.__init__', 'musy/chord.py'),
                            'musy.chord.PolyChord.__repr__': ('chord.html#polychord.__repr__', 'musy/chord.py'),
                            'musy.chord.PolyChord.invert': ('chord.html#polychord.invert', 'musy/chord.py'),
                            'musy.chord.PolyChord.to_frame': ('chord.html#polychord.to_frame', 'musy/chord.py')},
            'musy.midi': {'musy.midi.test_midi': ('midi.html#test_midi', 'musy/midi.py')},
            'musy.note': { 'musy.note.Interval': ('note.html#interval', 'musy/note.py'),
                           'musy.note.Interval.__abs__': ('note.html#interval.__abs__', 'musy/note.py'),
                           'musy.note.Interval.__add__': ('note.html#interval.__add__', 'musy/note.py'),
                           'musy.note.Interval.__eq__': ('note.html#interval.__eq__', 'musy/note.py'),
                           'musy.note.Interval.__ge__': ('note.html#interval.__ge__', 'musy/note.py'),
                           'musy.note.Interval.__gt__': ('note.html#interval.__gt__', 'musy/note.py'),
                           'musy.note.Interval.__index__': ('note.html#interval.__index__', 'musy/note.py'),
                           'musy.note.Interval.__init__': ('note.html#interval.__init__', 'musy/note.py'),
                           'musy.note.Interval.__int__': ('note.html#interval.__int__', 'musy/note.py'),
                           'musy.note.Interval.__le__': ('note.html#interval.__le__', 'musy/note.py'),
                           'musy.note.Interval.__lt__': ('note.html#interval.__lt__', 'musy/note.py'),
                           'musy.note.Interval.__ne__': ('note.html#interval.__ne__', 'musy/note.py'),
                           'musy.note.Interval.__repr__': ('note.html#interval.__repr__', 'musy/note.py'),
                           'musy.note.Interval.__sub__': ('note.html#interval.__sub__', 'musy/note.py'),
                           'musy.note.Interval.from_semitones': ('note.html#interval.from_semitones', 'musy/note.py'),
                           'musy.note.Interval.is_consonant': ('note.html#interval.is_consonant', 'musy/note.py'),
                           'musy.note.Interval.is_contextual': ('note.html#interval.is_contextual', 'musy/note.py'),
                           'musy.note.Interval.is_dissonant': ('note.html#interval.is_dissonant', 'musy/note.py'),
                           'musy.note.Interval.is_perfect': ('note.html#interval.is_perfect', 'musy/note.py'),
                           'musy.note.Interval.long': ('note.html#interval.long', 'musy/note.py'),
                           'musy.note.Interval.midi': ('note.html#interval.midi', 'musy/note.py'),
                           'musy.note.Interval.set_semitones': ('note.html#interval.set_semitones', 'musy/note.py'),
                           'musy.note.Interval.short': ('note.html#interval.short', 'musy/note.py'),
                           'musy.note.Interval.type': ('note.html#interval.type', 'musy/note.py'),
                           'musy.note.Note': ('note.html#note', 'musy/note.py'),
                           'musy.note.Note.__add__': ('note.html#note.__add__', 'musy/note.py'),
                           'musy.note.Note.__and__': ('note.html#note.__and__', 'musy/note.py'),
                           'musy.note.Note.__eq__': ('note.html#note.__eq__', 'musy/note.py'),
                           'musy.note.Note.__floordiv__': ('note.html#note.__floordiv__', 'musy/note.py'),
                           'musy.note.Note.__ge__': ('note.html#note.__ge__', 'musy/note.py'),
                           'musy.note.Note.__gt__': ('note.html#note.__gt__', 'musy/note.py'),
                           'musy.note.Note.__index__': ('note.html#note.__index__', 'musy/note.py'),
                           'musy.note.Note.__init__': ('note.html#note.__init__', 'musy/note.py'),
                           'musy.note.Note.__int__': ('note.html#note.__int__', 'musy/note.py'),
                           'musy.note.Note.__le__': ('note.html#note.__le__', 'musy/note.py'),
                           'musy.note.Note.__lt__': ('note.html#note.__lt__', 'musy/note.py'),
                           'musy.note.Note.__mod__': ('note.html#note.__mod__', 'musy/note.py'),
                           'musy.note.Note.__ne__': ('note.html#note.__ne__', 'musy/note.py'),
                           'musy.note.Note.__repr__': ('note.html#note.__repr__', 'musy/note.py'),
                           'musy.note.Note.__str__': ('note.html#note.__str__', 'musy/note.py'),
                           'musy.note.Note.__sub__': ('note.html#note.__sub__', 'musy/note.py'),
                           'musy.note.Note.from_int': ('note.html#note.from_int', 'musy/note.py'),
                           'musy.note.Note.from_midi': ('note.html#note.from_midi', 'musy/note.py'),
                           'musy.note.Note.get_audio_array': ('note.html#note.get_audio_array', 'musy/note.py'),
                           'musy.note.Note.get_audio_bytes': ('note.html#note.get_audio_bytes', 'musy/note.py'),
                           'musy.note.Note.interval': ('note.html#note.interval', 'musy/note.py'),
                           'musy.note.Note.major': ('note.html#note.major', 'musy/note.py'),
                           'musy.note.Note.midi': ('note.html#note.midi', 'musy/note.py'),
                           'musy.note.Note.minor': ('note.html#note.minor', 'musy/note.py'),
                           'musy.note.Note.play': ('note.html#note.play', 'musy/note.py')},
            'musy.scale': { 'musy.scale.Scale': ('scale.html#scale', 'musy/scale.py'),
                            'musy.scale.Scale.__eq__': ('scale.html#scale.__eq__', 'musy/scale.py'),
                            'musy.scale.Scale.__init__': ('scale.html#scale.__init__', 'musy/scale.py'),
                            'musy.scale.Scale.__iter__': ('scale.html#scale.__iter__', 'musy/scale.py'),
                            'musy.scale.Scale.__len__': ('scale.html#scale.__len__', 'musy/scale.py'),
                            'musy.scale.Scale.__ne__': ('scale.html#scale.__ne__', 'musy/scale.py'),
                            'musy.scale.Scale.__repr__': ('scale.html#scale.__repr__', 'musy/scale.py'),
                            'musy.scale.Scale._shift_abs_semitones': ('scale.html#scale._shift_abs_semitones', 'musy/scale.py'),
                            'musy.scale.Scale._shift_rel_semitones': ('scale.html#scale._shift_rel_semitones', 'musy/scale.py'),
                            'musy.scale.Scale.abs_interval_names': ('scale.html#scale.abs_interval_names', 'musy/scale.py'),
                            'musy.scale.Scale.abs_semitones': ('scale.html#scale.abs_semitones', 'musy/scale.py'),
                            'musy.scale.Scale.available_scales': ('scale.html#scale.available_scales', 'musy/scale.py'),
                            'musy.scale.Scale.from_intervals': ('scale.html#scale.from_intervals', 'musy/scale.py'),
                            'musy.scale.Scale.get_audio_array': ('scale.html#scale.get_audio_array', 'musy/scale.py'),
                            'musy.scale.Scale.get_diatonic_chords': ('scale.html#scale.get_diatonic_chords', 'musy/scale.py'),
                            'musy.scale.Scale.get_modes': ('scale.html#scale.get_modes', 'musy/scale.py'),
                            'musy.scale.Scale.get_notes': ('scale.html#scale.get_notes', 'musy/scale.py'),
                            'musy.scale.Scale.get_scale_names': ('scale.html#scale.get_scale_names', 'musy/scale.py'),
                            'musy.scale.Scale.get_sevenths': ('scale.html#scale.get_sevenths', 'musy/scale.py'),
                            'musy.scale.Scale.get_triads': ('scale.html#scale.get_triads', 'musy/scale.py'),
                            'musy.scale.Scale.play': ('scale.html#scale.play', 'musy/scale.py'),
                            'musy.scale.Scale.play_sevenths': ('scale.html#scale.play_sevenths', 'musy/scale.py'),
                            'musy.scale.Scale.play_triads': ('scale.html#scale.play_triads', 'musy/scale.py'),
                            'musy.scale.Scale.rel_interval_names': ('scale.html#scale.rel_interval_names', 'musy/scale.py'),
                            'musy.scale.Scale.rel_semitones': ('scale.html#scale.rel_semitones', 'musy/scale.py'),
                            'musy.scale.Scale.shift_intvals': ('scale.html#scale.shift_intvals', 'musy/scale.py'),
                            'musy.scale.Scale.to_frame': ('scale.html#scale.to_frame', 'musy/scale.py'),
                            'musy.scale.semi_to_intvals': ('scale.html#semi_to_intvals', 'musy/scale.py')},
            'musy.viz': { 'musy.viz.Guitar': ('viz.html#guitar', 'musy/viz.py'),
                          'musy.viz.Guitar.__ft__': ('viz.html#guitar.__ft__', 'musy/viz.py'),
                          'musy.viz.Piano': ('viz.html#piano', 'musy/viz.py'),
                          'musy.viz.Piano.__call__': ('viz.html#piano.__call__', 'musy/viz.py'),
                          'musy.viz.Piano.__ft__': ('viz.html#piano.__ft__', 'musy/viz.py')}}}
